/** 
 *   Thermometer TC6003P
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <sys/time.h>
#include "thermometer.h"
#include "my_stdio.h"

#define PRINT_REG(time_secs, temp1) \
        printf("%10.6f %5.1f\n", time_secs, temp1)

#define DIFTV(t1, t0) (\
        (double)(t1.tv_sec - t0.tv_sec) + \
        (double)(t1.tv_usec - t0.tv_usec)/1000000.0)


const char *help_text[] ={
#       ifdef VERBOSE_MODE
        "  -v\t\t - Verbose mode (incremental)",
#       endif
	"  -h\t\t - this Help",
	"  -a\t\t - print All temperatures",
	"  -d device\t - thermometer Device (def "THERMOMETER_DEVICE")",
	"  -s time\t - Saturation time to stop (sec)",
	"  -w time\t - Wait until saturation time is apply (sec)",
	""
};
	

void usage(char *prog)
{
	int i=0;

	fprintf(stderr, 
		"Use %s [-vha] [-d device (def: %s)] [-s time] [-w time]\n", 
		prog, THERMOMETER_DEVICE);
	while((char) *help_text[i] != 0)
                fprintf(stderr,"%s.\n",help_text[i++]);
        fprintf(stderr,"\n");
};


int main(int argc, char * const argv[])
{
	int fd;
	int op;
	double temp0, temp1;
	double now_t0;
	char *device = THERMOMETER_DEVICE;
	struct timeval t0;
	struct timeval last;
	struct timeval now;
	struct timeval prev;
	int done = 0;
	int print;
	int skip = 0;
	int stop_flag = 0;
	int print_all_flag = 0;
	time_t wait_until_stop = 0;
	time_t saturation_time = 0;
#ifdef VERBOSE_MODE
	verbose_flag = PERR;
#endif
	while  ((op = getopt(argc, argv, "vhd:s:w:a")) != EOF)
		switch( op ){
#ifdef VERBOSE_MODE
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#endif
		case 'd':
			device = optarg;
			break;
		case 's':
			stop_flag = 1;
			saturation_time = (time_t) atol(optarg);
			break;
		case 'w':
			stop_flag = 1;
			wait_until_stop = (time_t) atol(optarg);
			break;
		case 'a':
			print_all_flag = 1;
			break;
		case 'h':
		default:
			usage(argv[0]);
			exit(-1);
		}
	
	if ((fd = thermometer_open(device)) < 0) {
		perror("Thermometer open");
		return -1;
	}	
	if (thermometer_get(fd, &temp0) != 0) {
		fprintf(stderr, "Error\n");
		return -1;
	}
	gettimeofday(&t0, NULL);
	memcpy(&last, &t0, sizeof(struct timeval));
	printf("# DATE = %ld.%6ld\n", t0.tv_sec, t0.tv_usec);
	printf("# device = %s\n", device);
	if ( stop_flag ) {
		printf("# s = %ld\n", saturation_time );
		printf("# w = %ld\n", wait_until_stop );
	}
	if (print_all_flag) {
		printf("# a = on\n");
	}
	printf("# Time     Temperature\n");
	PRINT_REG(0.0, temp0);
	
	while(!done) {
		if (thermometer_get(fd, &temp1) != 0) {
			fprintf(stderr, "Error\n");
			return -1;
		}
		gettimeofday(&now, NULL);
		now_t0 = DIFTV(now, t0);
		print = 0;
		if ( temp1 != temp0 || print_all_flag ) {
			if (skip > 0) {
				PRINT_REG(DIFTV(prev, t0), temp0);
			}
			memcpy(&last, &now, sizeof(struct timeval));
			temp0 = temp1;
			PRINT_REG(now_t0, temp1);
			print = 1;
			skip = 0;
		} 
		else {
			memcpy(&prev, &now, sizeof(struct timeval));
			skip++;
		}
		if (stop_flag &&  
		    (time_t) now_t0 >= wait_until_stop &&
		    (time_t) DIFTV(now, last) >= saturation_time   ) {
			done = 1;
			if (!print) PRINT_REG(now_t0, temp1);
		}
	}
	thermometer_close(fd);
	fprintf(stderr, "%c", 0x7);
	
	return 0;
}

